@@ -85,6 +85,9 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=
 
     arg7 (Boolean): ต้องการ clean หัวของคอลัมน์หรือไม่ เช่น ':Close' เป็นต้น
     """ 
+    print("ยกเลิกใน version 155530 ให้ใช้ loadHistData_v2 ทดแทน\nเช่น sx.loadHistData_v2(symbol='aot',start='2020-01-01',end='2020-12-31')")
+    return 0
+
     if(end==""):
      now = datetime.datetime.now()
      end = (str(now.year).strip()+"-"+str(now.month).strip()+"-"+str(now.day).strip())
@@ -164,11 +167,11 @@ def loadHistData(symbol,start,end="",Volume=False,OHLC=False,US=False,col_clean=
  
     #จบส่วนของเช็ค str
 
- 
+    
     if(qs!=0): #คือมีชุด query String ของหุ้นรายตัว*  ต่อมาสำเร็จ กรณีไม่สำเร็จเช่น ["^set","^set50"]
      try:
       df = loaddata.get(qs,start=start,end=endDate)
-      
+      print(df)
       if(len(symbol_stock_only)==0):
           symbol_stock_only = symbol
         
@@ -452,6 +455,7 @@ def RelativeStrength_hist(dateStart,dateStop,setUni,plotState,sp_data=None,numCo
     else:
       print(symbol,end="")  
     
+
     df = loaddata.get(qs,start=dateStart,end=dateStop)
     df.columns = [symbol]
     
@@ -660,6 +664,166 @@ def RelativeStrengthRank(dateStart,dateStop,setUni,dataset=0,viewlog=False):
   return busket 
 
 
+
+exec("from "+prefix+"loadHistData_v2 import loadHistData_v2")
+
+#%config InlineBackend.figure_format ='retina'
+def RelativeStrength_hist_with_progress1(dateStart,dateStop,setUni,plotState,sp_data=None,numColPerRow=10,market=None):
+
+ if(market==None):
+   market = setUni
+
+ symbol = "^"+market.value
+ #print(symbol)
+ tdex = loadHistData(symbol,start=dateStart,end=dateStop) 
+ tdex.columns = ['close']
+ 
+
+ tdexbusket = pd.DataFrame({"startDate":tdex.iloc[0],"endDate":tdex.iloc[-1],"symbol":"tdex"})
+ tdexbusket = tdexbusket.set_index('symbol')  
+ 
+ #ดึงรายชื่อหุ้นใน set50 หรือ set100  
+ #busketDS = getMemberOfIndex(indexMarket.SET50)   
+ busketDS = getMemberOfIndex(setUni)
+ #print(type(busketDS))
+
+ if(sp_data!=None):     
+   key_index = np.array(range(len(busketDS),len(busketDS)+len(sp_data)))
+   new_data = pd.DataFrame({"symbol":sp_data},index=key_index)
+   #busketDS  = busketDS.append(new_data)
+   busketDS = pd.concat([new_data, busketDS], ignore_index=True)
+   busketDS = busketDS.sort_values('symbol',ascending=False)
+
+ #print(busketDS)
+ #return 0
+ busket = pd.DataFrame()
+ busketDS = np.array(busketDS)
+ cnt = len(busketDS)
+ for i in range(cnt):
+    symbol = busketDS[i][0]
+   
+    qs = crateQueryString([symbol])
+    #if(i<cnt-1):
+    #  print(symbol,end=",")
+    #else:
+    #  print(symbol,end="")  
+    
+    print(qs)
+
+    #fix bug กณณี load ไม่ขึ้น 
+    try:
+      df = loaddata.get(qs,start=dateStart,end=dateStop)
+      df.columns = [symbol]
+    except:  
+
+      df = loadHistData_v2(symbol=symbol,start=dateStart,end=dateStop)[['Close']]
+      df.columns = [symbol]
+    
+    yield df 
+  
+def RelativeStrength_hist_with_progress2(basket,setUni,dateStart='2021-01-01',dateStop='2021-06-30',market=None):
+ if(market==None):
+   market = setUni
+
+ symbol = "^"+market.value
+ print(symbol)
+ tdex = loadHistData(symbol,start=dateStart,end=dateStop) 
+ tdex.columns = ['close']
+
+ tmpRSX = pd.DataFrame()
+ dayCnt = len(basket)
+ stockCnt = len(basket.columns) 
+
+ for i in range(1,dayCnt): #each day
+  rs_ds = []
+  for j in range(stockCnt): #each stock
+    beginStockPrice = basket[basket.columns[j]].iloc[0]
+    endStockPrice = basket[basket.columns[j]].iloc[i]
+    
+    #print(tdex)
+    #return 0
+    
+    beginIndex = tdex.iloc[0].values[0]
+    endIndex = tdex.iloc[i].values[0]
+
+    #print(endStockPrice,beginStockPrice,beginIndex,endIndex)
+    #print(beginIndex,endIndex)
+    #return 0
+
+    rs = ((endStockPrice/beginStockPrice) /  (endIndex/beginIndex))
+    rs_ds.append(rs)
+
+  if(i==1):
+   tmpRS = pd.DataFrame(rs_ds).T
+   tmpRS.columns = basket.columns
+   tmpRS['Date'] = tdex.index[i]
+
+   tmpRSX = tmpRS.set_index("Date")  
+  else:
+   tmpRS = pd.DataFrame(rs_ds).T
+   tmpRS.columns = basket.columns
+   tmpRS['Date'] = tdex.index[i]
+   tmpRS = tmpRS.set_index("Date") 
+   tmpRSX = tmpRSX.append(tmpRS) 
+
+ print("..",end="")
+ print(" End Process")   
+
+ 
+ tmpRSX_rank = pd.DataFrame()
+ for i in range(len(tmpRSX)):
+
+  t = pd.DataFrame(tmpRSX.iloc[i].sort_values(ascending=False))
+  t['RANK'] = list(range(len(t), 0,-1))
+  t.columns = ['RS','RANK']
+  maxRank = t['RANK'].max()
+  minRank = t['RANK'].min()
+  maxScope = 99
+  minScope = 1
+  S = (minScope-maxScope)/(minRank-maxRank)
+  Int = minScope - S*minRank
+  t['RS_Rank'] = t['RANK']*S+Int
+    
+  if(i==0):   
+   tmpRS_rank = pd.DataFrame(t['RS_Rank'])
+   tmpRS_rank = tmpRS_rank.sort_index()
+   tmpRS_rank = tmpRS_rank.T
+   tmpRS_rank['Date'] = tmpRSX.index[i]
+   tmpRSX_rank = tmpRS_rank.set_index("Date")
+  else:
+   tmpRS_rank = pd.DataFrame(t['RS_Rank'])
+   tmpRS_rank = tmpRS_rank.sort_index()
+   tmpRS_rank = tmpRS_rank.T
+   tmpRS_rank['Date'] = tmpRSX.index[i]  
+   tmpRS_rank = tmpRS_rank.set_index("Date") 
+
+   tmpRSX_rank = tmpRSX_rank.append(tmpRS_rank)  
+
+
+ 
+
+ tmpRSX_rank.index = tmpRSX_rank.index.date #เทคนิค เอาเวลาออกจาก date คือตัด HH:MM:SS ออก   
+ return tmpRSX_rank  
+
+
+#หาอัตราการเปรียบเปลี่ยนราคาเปรียบเทียบกับดัชนี
+#Function RelativeStrengthRank(dateStart,setUni)
+#parameter 
+#  setUni ตัวแทนทั้งหมดที่จะนำไปเปรียบเทียบ  ตอนนี้ส่งค่าเป็น 'SET50' หรือ 'SET100'
+#  dateStart วันที่เริ่มต้น โดยวันที่สุดท้ายจะเป็นวันปัจจุบัน เช่น 
+
+#return DataFrame ของ RelativeStrength Rank คะแนน 1-99
+
+
+#หาอัตราการเปรียบเปลี่ยนราคาเปรียบเทียบกับดัชนี
+#Function RelativeStrengthRank(dateStart,setUni)
+#parameter 
+#  setUni ตัวแทนทั้งหมดที่จะนำไปเปรียบเทียบ  ตอนนี้ส่งค่าเป็น 'SET50' หรือ 'SET100'
+#  dateStart วันที่เริ่มต้น โดยวันที่สุดท้ายจะเป็นวันปัจจุบัน เช่น 
+
+#return DataFrame ของ RelativeStrength Rank คะแนน 1-99
+
+
 ############################# find_zone ############################
 def plotNormal(df,priceNow,bins):
     fig, ax = plt.subplots(figsize=(10,8))
@@ -1400,9 +1564,18 @@ def getFinanceRatioProcess(symbol):
     return 0
     
    #มีการปรับใช้ภาษาอังกฤษแทน เพราะว่ากังวลเรื่อง font TH ที่ต้อง plugin เข้าไป Version 0.15496
-   g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})[22].find_all("td")
-   if not "Annualized: ROE, ROA" in str(g_data):
-    g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})[21].find_all("td")
+
+   #### หา index #### content อยู่ช่องไหน
+   ct = soup.find_all("table", {"class" : "table-factsheet-padding0"})
+   i_index = 0
+   for i in ct:
+    if("Annualized: ROE, ROA" in i.text):
+      m_index = i_index
+      break
+    i_index+=1
+   ####################################
+
+   g_data = soup.find_all("table", {"class" : "table-factsheet-padding0"})[i_index].find_all("td")
     
    if not "Annualized: ROE, ROA" in str(g_data):
       print("ไม่พบข้อมูล")
@@ -1421,6 +1594,7 @@ def getFinanceRatioProcess(symbol):
     s = str(g_data[i].text.strip())
     s = s.rstrip()
     s = s.lstrip()
+    s = s.replace(',','')
     
  
     
@@ -1884,9 +2058,10 @@ def trimdata(s):
   return s
 
 def listStockInSector(sample_in_sector):
+ sample_in_sector = sample_in_sector.replace("&","%26")  
  headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}   
  r = requests.post("http://www.settrade.com/C04_08_stock_sectorcomparison_p1.jsp?txtSymbol="+sample_in_sector,headers=headers)
- r.content
+ #r.content
  soup = BeautifulSoup(r.content, "lxml")  
  g_data = soup.find_all("table", {"class" : "table table-info"})
  
@@ -1908,13 +2083,27 @@ def listStockInSector(sample_in_sector):
 
 ######################################
 def getdata(symbol,returntype="ar"):
- symboltmp = symbol.replace('&', '%26') 
+ symboltmp = symbol.replace('&', '+%26+') 
  datasource = []   
- headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"}      
- r = requests.post("http://www.settrade.com/C04_06_stock_financial_p1.jsp?txtSymbol="+symboltmp,headers)
- r.content
+ headers = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"} 
+ #url = "http://www.settrade.com/C04_06_stock_financial_p1.jsp?txtSymbol="+symboltmp  
+
+ #แก้ไขใน version 2021-10-เมษายน เพราะ url เดิมไม่ได้คืนค่ามาให้ version 0.155530
+ url = 'https://www.settrade.com/sims/dat/finstmt/files/'+symboltmp+'_th_TH.html'
+
+ #แก้ไขใน version 2021-10-เมษายน เปลี่ยนเป็น get version 0.155530
+ r = requests.get(url,headers)
+ #r.content
+ 
  soup = BeautifulSoup(r.content, "lxml")  
- g_data = soup.find_all("table", {"class" : "table table-hover table-info"})[0].find_all("tr")
+
+ try:
+  g_data = soup.find_all("table", {"class" : "table table-hover table-info"})[0].find_all("tr")
+ except:
+  #case S&J
+  return 0
+ #################    
+
  col = []
  datasource.append(symbol)
   
@@ -2121,11 +2310,10 @@ def ar2df(ar):
 #แก้ใน version 0.1544
 def getFundamentalInSector(sample_in_sector,progress=True):
  datasource = []
- p = listStockInSector(sample_in_sector=sample_in_sector)    
+ p = listStockInSector(sample_in_sector=sample_in_sector) 
+ #print(p)   
  for k in range(len(p)):
-    
     r = p.loc[k].values
- 
     s = getdata(str(r[0]))
     if(s!=0):
      if(progress==True): 
